#!/bin/bash
#
# (c) astx (at) aws-it.at, 2018 - 2019
# and (c) others - see DIYaudio thread https://www.diyaudio.com/forums/software-tools/331656-installing-using-ngspice-opensource-simulator.html
#
# $Id: runme.sh 1590 2019-02-02 16:36:03Z sta $
#
echo ""; echo ""
echo "#-----------------------------------------------------------------------"
echo "NOTE: For successful run you need at least ngspice 30"
echo "After simulation run you can further explore the result data"
echo "Use 'quit' to exit simulation"
echo ""
echo "#-----------------------------------------------------------------------"
echo "If you want to run opamp stepping simulation now enter 'y' <ENTER>"
echo "#-----------------------------------------------------------------------"

read a
if [ "$a" == "y" ]; then

	# change this to your preferred editor. vi nano joe ...
	export SPICE_EDITOR="joe"

	# temporary net list files
	mynet="wingspread"

	chead="* ng_script\n.control\n"
	cend="\n.endc\n"

	echo -en "${chead}" > load_result_files0.inc
	echo -en "${chead}" > load_result_files1.inc
	echo -en "${chead}" > load_result_files2.inc

	# set variable arrays used in output data
	plots=''
	vbias=''

	loop=0
	count=1

	RG=15
	SUPPLY=50
	REM=0.1

	# we do a bias parameter stepping to get wingspread data for different bias current values
	VBIAS[0]=4.60
	VBIAS[1]=4.65
	VBIAS[2]=4.70
	VBIAS[3]=4.75
	VBIAS[4]=4.80

	# STEPPING bias voltage for all available array elements
	while [ $loop -lt ${#VBIAS[*]} ];
	do
		echo "LOOP $count"
		# load schematic and create a fresh netlist
		gnetlist -g spice-sdb -o $mynet.net $mynet.sch

		sed -i -r -e "s/\%RG\%/${RG}/g" $mynet.net
		sed -i -r -e "s/\%REM\%/${REM}/g" $mynet.net
		sed -i -r -e "s/\%SUPPLY\%/${SUPPLY}/g" $mynet.net
		sed -i -r -e "s/\%VBIAS\%/${VBIAS[$loop]}/g" $mynet.net

		# e.g. add/set some circuit params or options be4 simulation run
		echo -en "* ng_script\n.options savecurrents\n.options numdgt=8\n.control\nset myloop=\"${loop}\"\n"	> $mynet.inc
		echo "echo \"Starting with loop Nr.: \$myloop\""	>> $mynet.inc
echo "\
dc Vin -15V +15V 0.05
let dd=deriv(output)
settype notype dd
meas dc cbias find @r5[i] when v(output)=0
write output_${loop}.data dd cbias
quit
" >> $mynet.inc

		echo -en "\n.endc\n\n"					>> $mynet.inc

		ngspice $mynet.net $mynet.inc
		# generate load result files listing
		echo "load output_${loop}.data" >> load_result_files1.inc
		plots="${plots}dc${count} "
		vbias="${vbias}${VBIAS[$loop]} "
		count=$(($count+1))
		loop=$(($loop+1))
	done

	echo -en "${cend}" >> load_result_files1.inc
	# set variables 
	echo "set wgplots=( ${plots})" >> load_result_files0.inc
	echo "set vbias=( ${vbias})" >> load_result_files0.inc
	echo -en "${cend}" >> load_result_files0.inc

	# load (no netlist!) results only from stepping and display e.g. fft, fourier and input, output voltages
echo "* ng_script
.include load_result_files0.inc
.include load_result_files1.inc
.control

	set hcopywidth=1920
	set hcopyheight=1080

	* plot all stepped results in one plot
	set mypl1 = ( )
	set nolc_biastxt=""
	let loop=0

	foreach pl \$wgplots
		let bc=nint({\$pl}.cbias*1000)
		set nolc_biastxt=\"\$nolc_biastxt \$&bc\"

		set mypl1 = ( \$mypl1 {\$pl}.dd )

		* as long as we stay in last plot, we get no problem with our loop variable 
		let loop=loop+1
		* looks like text arrays via set start at index 1 instead of 0 ?
		let bv=2*\$vbias[$&loop]
		echo bias current: \$&bc mA when bias voltage is \$&bv V
	end
	plot \$mypl1 xlabel Input ylabel Gain title \"Wingspread plot for bias \$nolc_biastxt mA\"

	* NOTE: this needs a ngspice patch to get nice png output where gnuplot makes a auto exit
	set gnuplot_terminal=png/quit
	gnuplot wingspread_plot_result \$mypl1 xlabel \"Input V\"  ylabel Gain title \"Wingspread plot for bias \$nolc_biastxt mA\"

.endc
" > $mynet.inc2

	ngspice $mynet.inc2

	# reset terminal if ngspice terminates unexpectedly
	#	reset
fi
